/*
 * PHOSPHOR_MIDI
 *
 * Created: March 2019
 * Author: Joris Wegner (joriswegner.de)
 * 
 * !!! THIS IS THE   U N T E S T E D   CODE VERSION FOR A VARIANT WITH A TLC5940 AS SMD PACKAGE !!!
 * 
 * This sketch controls the PHOSPHOR analog video distortion unit.
 * A MIDI ON command on CHANNEL 1 selects one of the patches stored in the matrix below to trigger cool video glitch effects.
 * Use the PHOSPHOR_PROGRAMMER Processing sketch together with the PHOSPHOR_PROGRAMMER Arduino sketch to create patches, then paste them into
 * the matrix below. Change numberofpatches accordingly.
 * 
 * The first matrix entry should be left as it is. Sending a C4 note on MIDI CHANNEL 1 will then disable all effects.
 * The next entries can be filled with custom patches instead. Every next entry is triggered with the next semitone.
 * 
 * This sketch only reacts to MIDI ON commands. This way, you can send the desired note multiple times in a short period
 * to reliably change the patches.
 * 
 * There are no presets because I believe that the tolerances in the hardware are too large to make patches that work well on all devices.
 * 
 * USE THIS DEVICE AT YOUR OWN RISK. Using the original software to create and use patches as well as the recommended
 * parts, the unit should use much less current then a usb port can deliver. The video circuitry should be isolated 
 * well enough from the controller circuit that is connected to the computer. However, I am not responsible for any damage
 * that could occur while using this device.
 * 
 * 
 * 
 * CHANGE PATCHES BY SENDING NOTE C4 OR HIGHER ON MIDI CHANNEL 1
 *
 */ 


#include <MIDIUSB.h>
#include "SparkFun_Tlc5940.h"

bool debug = false;              // Set to true for Serial messages

int patch;                      // The patch that is to be activated
int activepatch = -1;           // The currently active patch
const int numberofpatches = 13; // The total number of patches (change this if you paste in more)

// Paste your patches from the
// processing sketch into this matrix

int patches[numberofpatches][11] = {
  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }, // C4 -- Patch 0: Leave this unchanged to disable all effects with C4
  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }, // C#4
  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }, // D4
  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }, // D#4
  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }, // E4
  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }, // F4
  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }, // F#4
  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }, // G4
  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }, // G#4
  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }, // A4
  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }, // A#4
  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }, // B4
  { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }, // C5 etc.
};

int led = 14;                   // Signal LED lights when receiving a MIDI command
int loops = 500;                // For how many program loops the signal LED stays lit after receiving a MIDI command
int loopsleft = 0;              // How many loops are left until the LED is switched off

void setup() {
  Serial.begin(115200);
  Tlc.init();

  // This is for testing purposes
  if (debug) {
    digitalWrite(led, LOW);
    delay(3000);
    Serial.println("INIT TEST");
    Serial.println(" ");
    noteOn(1, 72, 126);  // Wrong channel
    noteOn(0, 72, 126);  // Patch 0
    noteOn(0, 59, 126);  // Too low
    noteOn(0, 100, 126); // Too high
    noteOn(0, 73, 126);  // Patch 1
    noteOn(0, 73, 126);  // Patch 1, no change
    noteOn(0, 74, 126);  // Patch 2
    Serial.println("END INIT TEST");
  }
}

// Is activated by noteOn commands, then selects the preset
// from the matrix and sets the LEDs accordingly
void noteOn(byte channel, byte pitch, byte velocity) {
  if (channel == 0) {                    // If channel = 1
    if (pitch >= 72) {                   // If note higher than or equal to C4
      patch = pitch - 72;                // C4 is patch 0
      if (debug) {
        Serial.print("PATCH: ");
        Serial.println(patch);
      }
      if (patch <= numberofpatches) {    // If the note is lower/equal to the number of patches available
        if (patch != activepatch) {      // If patch has changed

          if (debug) Serial.println("SET TLC");

          Tlc.set(2,  patches[patch][0]); //4 These are the original Pins for the DIP package
          Tlc.set(1,  patches[patch][1]); //1
          Tlc.set(5,  patches[patch][2]); //5
          Tlc.set(3,  patches[patch][3]); //2
          Tlc.set(7,  patches[patch][4]); //9
          Tlc.set(9,  patches[patch][5]); //11
          Tlc.set(8,  patches[patch][6]); //14
          Tlc.set(4,  patches[patch][7]); //3
          Tlc.set(10, patches[patch][8]); //13
          Tlc.set(11, patches[patch][9]); //12
          Tlc.set(6,  patches[patch][10]); //10
          Tlc.update();

          activepatch = patch;
          
          if (debug) {
            Serial.print("FIRST NUMBER IN PATCH: ");
            Serial.println(patches[patch][0]);
          }
          if (debug) {
            Serial.print("ACTIVE PATCH: ");
            Serial.println(activepatch);
          }


        } else {
          if (debug) Serial.println("NO PATCH CHANGE");
        }
      } else {
        if (debug) Serial.println("NOTE TOO HIGH");
      }
    } else {
      if (debug) Serial.println("NOTE TOO LOW, MUST BE >= C4");
    }
  } else {
    if (debug) Serial.println("NOT CHANNEL 1");
  }
  
  if (debug) Serial.println(" ");
}


// The program does not use NOTE OFF commands
// Send Note C4 for Patch 0 instead to deactivate all effects
void noteOff(byte channel, byte pitch, byte velocity) {
  if (debug) Serial.println("NOTE OFF");
}

// The program does not use CONTROL CHANGE commands
void controlChange(byte channel, byte control, byte value) {
  if (debug) Serial.println("CTRL CHANGE");
}


// Recieve MIDI commands
void loop() {
  midiEventPacket_t rx = MidiUSB.read();
  switch (rx.header) {
    case 0:
      //if (debug) Serial.println("0");
      if(loopsleft > 0){
        loopsleft = loopsleft - 1;
      }
      break;             //No pending events

    case 0x9:
      noteOn(
        rx.byte1 & 0xF,  //channel
        rx.byte2,        //pitch
        rx.byte3         //velocity
      );
      loopsleft = loops;
      break;

    case 0x8:
      noteOff(
        rx.byte1 & 0xF,  //channel
        rx.byte2,        //pitch
        rx.byte3         //velocity
      );
      loopsleft = loops;
      break;

    case 0xB:
      controlChange(
        rx.byte1 & 0xF,  //channel
        rx.byte2,        //control
        rx.byte3         //value
      );
      loopsleft = loops;
      break;

    default:
      if (debug) {
        Serial.print("Unhandled MIDI message: ");
        Serial.print(rx.header, HEX);
        Serial.print("-");
        Serial.print(rx.byte1, HEX);
        Serial.print("-");
        Serial.print(rx.byte2, HEX);
        Serial.print("-");
        Serial.println(rx.byte3, HEX);
      }
  }

  if(loopsleft>0){
    digitalWrite(led, HIGH);
  } else {
    digitalWrite(led, LOW);
  }
}
